(**************************************************************************)
(*  The CDuce interpreter                                                 *)
(*  Alain.Frisch@ens.fr                                                   *)
(*  Copyright 2003 (see LICENSE for details)                              *)
(**************************************************************************)

ifdef ML_INTERFACE then
type error =
  | Undefined_value of string * string
  | Type_mismatch of string * Ml_ocaml.Type.t * Ml_cduce.Type.t * Ml_cduce.Type.t

ifdef ML_INTERFACE then
exception Error of error

ifdef ML_INTERFACE then
let error e = raise ( Error e )

ifdef ML_INTERFACE then
let report_error = function
  | Undefined_value ( file, func ) ->
      Format.eprintf "cduce2ocaml : error in file %s.cmi:@." file;
      Format.eprintf "Value %s has no counterpart in file %s.cdo@." func file
  | Type_mismatch ( file, ml_t, ml_cd_t, cd_t ) ->
      Format.eprintf "cduce2ocaml : error in file %s.cmi:@." file;
      Ml_cduce.Type.print Format.err_formatter cd_t;
      Format.eprintf "\nis not a subtype of@.";
      Ml_cduce.Type.print Format.err_formatter ml_cd_t;
      Format.eprintf "\nwhich is the canonical translation of@.";
      Ml_ocaml.Type.print Format.err_formatter ( true, [ ml_t ] )

(***********************************************************************************)

ifdef ML_INTERFACE then
let run ml_cu cd_cu =
  let file = Ml_cduce.CompUnit.module_name cd_cu in
  String.set file 0 ( Char.lowercase ( String.get file 0 ) );
  try Ml_ocaml.CompUnit.iter (
    fun ( _, list ) -> match list with
      | [ { Ml_types.ml_kind = Ml_types.ML_value } as ml_t ] -> (
	  try
	    let cd_type = Ml_cduce.Type.from_ocaml ml_t
	    and cd_base = Ml_cduce.CompUnit.find_value cd_cu ml_t.Ml_types.ml_name in
	    if not ( Ml_cduce.Type.is_subtype cd_base cd_type )
	    then error ( Type_mismatch ( file, ml_t, cd_type, cd_base ) );
	  with Not_found ->
	    error ( Undefined_value ( file, ml_t.Ml_types.ml_name ) ) )
      | _ -> () 
  ) ml_cu
  with Error e -> report_error e; exit 1
