(**************************************************************************)
(*  The CDuce interpreter                                                 *)
(*  Alain.Frisch@ens.fr                                                   *)
(*  Copyright 2003 (see LICENSE for details)                              *)
(**************************************************************************)

open Location
type type_fun = Types.t -> bool -> Types.t

module Unary = struct
  module Op = struct
    type t = (loc -> type_fun -> type_fun) * (Value.t -> Value.t)
  end
  module Proxy = Custom.Proxy(Custom.String)(Typer)(Op)
  include Pool.NoHash(Proxy)

  let register name make typ run ser deser =
    Proxy.register name make
      { Proxy.content = (fun x -> (typ x, run x));
	Proxy.serialize = ser;
	Proxy.deserialize = deser };;
      
  Typer.mk_unary_op := (fun name env -> mk (Proxy.instantiate name env));;
  Typer.typ_unary_op := (fun i -> fst (Proxy.content (value i)));;
  Eval.eval_unary_op := (fun i -> snd (Proxy.content (value i)));;
  Lambda.Put.unary_op := serialize;;
  Lambda.Get.unary_op := deserialize;;
end

module Binary = struct
  module Op = struct
    type t = (loc -> type_fun -> type_fun -> type_fun) * 
	(Value.t -> Value.t -> Value.t)
  end
  module Proxy = Custom.Proxy(Custom.String)(Typer)(Op)
  include Pool.NoHash(Proxy)

  let register name make typ run ser deser =
    Proxy.register name make
      { Proxy.content = (fun x -> (typ x, run x));
	Proxy.serialize = ser;
	Proxy.deserialize = deser };;
      
  Typer.mk_binary_op := (fun name env -> mk (Proxy.instantiate name env));;
  Typer.typ_binary_op := (fun i -> fst (Proxy.content (value i)));;
  Eval.eval_binary_op := (fun i -> snd (Proxy.content (value i)));;
  Lambda.Put.binary_op := serialize;;
  Lambda.Get.binary_op := deserialize;;
end
