(**************************************************************************)
(*  The CDuce interpreter                                                 *)
(*  Alain.Frisch@ens.fr                                                   *)
(*  Copyright 2003 (see LICENSE for details)                              *)
(**************************************************************************)

type url = Filename of string | Url of string

ifdef CURL then
  let curl_support = true
else
  let curl_support = false

ifdef NETCLIENT then
  let netclient_support = true
else
  let netclient_support = false

let is_url s = 
  try let _ = Neturl.extract_url_scheme s in true
  with Neturl.Malformed_URL -> false
	
ifdef CURL then
  let load_url s =
    let buff = Buffer.create 4096 in 
    let conn = Curl.init () in
    Curl.set_url conn s;
    Curl.set_writefunction conn (Buffer.add_string buff);
    Curl.perform conn;
    Buffer.contents buff
else
ifdef NETCLIENT then
  let load_url s =
    match  Neturl.extract_url_scheme s with
      | "http" -> 
	  (try Http_client.Convenience.http_get s
	   with 
	     | Http_client.Bad_message s ->
		 let msg = Printf.sprintf "Netclient. Bad http answer: %s" s in
		 raise (Location.Generic msg)
	     | Http_client.Http_error (n,s) ->
		 let msg = Printf.sprintf "Netclient. Http error %i: %s" n s in
		 raise (Location.Generic msg)
	     | Http_client.No_reply ->
		 raise (Location.Generic "Netclient. No reply")
	     | Http_client.Http_protocol exn ->
		 let msg = Printf.sprintf "Netclient. %s" 
			     (Printexc.to_string exn) in
		 raise (Location.Generic msg)
	  )
      | "file" ->
	  raise (Location.Generic 
		   "FIXME: write in url.ml the code so that netclient \
                    handle file:// protocol")
      | sc -> 
	  let msg = 
	    Printf.sprintf "Error: netclient does not handle the %s protocol" sc
	  in
	  raise (Location.Generic msg) 
else
  let load_url s =   
    let msg =  
        Printf.sprintf 
	  "Error \"%s\": \nTo fetch external URLs, you need to compile CDuce with curl and/or netclient" s 
    in
    raise (Location.Generic msg) 

let process s =
  if is_url s then Url (load_url s)
  else Filename s
